#!/bin/bash
#
# ascan-ox -- Generate ascan plots
# for Scion HF radar raw data files. 
# Terry Bullett
# University of Colorado
#
# Version 2.06  12Mar11
# useage:
# ascan-ox {switches} DATFILE
#
# Requires:
#    msttcorefonts (included)
#    bc
#    riq-ionogram version 2.05+
#    station_settings.sh
#    dump_headers
#    buthead 
# 
# Version 2.06
#
export GDFONTPATH=/usr/share/fonts/truetype/msttcorefonts/
#export GDFONTPATH=$HOME/lib/msttcorefonts/

APPNAME=ascan-ox
TFMT='%F %T %Z'
REALTIME=$(date -u +"$TFMT" )
YYYY=$(date -u +%Y)
DDD=$(date -u +%j)
LOGFILE="$HOME/log/$APPNAME-$YYYY-$DDD.log"
ERRLOG="$HOME/log/$APPNAME-$YYYY-$DDD.errlog"
TMPDIR="$HOME/tmp/$APPNAME"
mkdir -p $TMPDIR
#
# User-adjustable items.  
# Maximum plot values may be over-written by the station_settings.sh function.
SIZE="1024,768"
RANGESPAN="[50:450]"
XTICS=25
MXTICS=5
PWRMIN=0
PWRMAX=60
SNRMAX=50
OXMAX=50
#
RIQARGS="-i -X "
declare -i IFRQ NLINE CLEAN SMASK UOR

CLEAN=$((0))
UOR=$((0))
SMASK=$((2#11111111))
# Process command line switches

while getopts 'chm:n:o:r:t:' OPTION
do
   case $OPTION in
      c) CLEAN=$((1)) 
         ;;
      h)
         echo "ascan-ox Version 2.06"
	 echo "Plot VIPIR ionogram data in ascan mode"
         echo "Optional switches"
         echo "-h Print this help message"
         echo "-r RANGESPAN Change the range span"
	 echo "-m MASK  Set the receiver mask to MASK"
	 echo "-n NOISE  Adjust noise level"
         echo "-o OUTDIR  Set output directory, otherwise same as INFILE"
         echo "-R RANGE0  Manually set the range calibration"
         echo "-t {title} Add to the title of the plot"
         echo "-U URSI   URSI Code over-ride"
	 echo ""
	 exit 0 
	 ;;
      f) RANGESPAN="$OPTARG"
         ;;
      m) MASK=$(($OPTARG))
         ;;
      n) NOISE="$OPTARG"
         ;;
      o) OUTDIR="$OPTARG"
         ;;
      R) RANGE0="-R0 $OPTARG"
         ;;
      t) TITLE="$OPTARG"
         ;;
      U) URSI="$OPTARG"
         UOR=$((1))
         ;;
   esac
done
#

shift $((OPTIND - 1))
FILELIST="$@"
for INFILE in $FILELIST ; do 
   REALTIME=$(date -u +"$TFMT" )
   if [ -z $INFILE ] ; then 
      echo "$REALTIME - $APPNAME - FATAL - Input file is undefined" >> $LOGFILE 
      exit -1
   fi
   if [ ! -r $INFILE ] ; then 
      echo "$REALTIME - $APPNAME - FATAL - Input file $INFILE does not exist or is non-readable" >> $LOGFILE 
      exit -2
   fi
#
   if [ -z $OUTDIR ] ; then
      DIR=$(dirname $INFILE)
   else
      DIR=$OUTDIR
   fi
#
#  Determine if the data file is compressed.
#  ZIP compression is not supported, but it could be.
#  need to preserve the ZIP file.
#
   BZIP2=$(file $INFILE | grep bzip2)
   GZIP=$(file $INFILE | grep gzip)
#
#  Determine file names
   if [ -n "$BZIP2" ] ; then 
         RIQFILE="$(dirname $INFILE)/$(basename $INFILE .bz2)" 
   elif [ -n "$GZIP" ] ; then
      RIQFILE="$(dirname $INFILE)/$(basename $INFILE .gz)"
   else
      RIQFILE="$INFILE"
   fi 
   BFN=$(basename $RIQFILE .RIQ)
#  For v0 data files with no station ID in the filename
   if (( $UOR )) ; then
      URSI="$URSI"
   else 
      URSI=${BFN:0:5}
   fi 
#  Determine the measurement time from the standard filename
   YEAR=${BFN:6:4}
   DAY=${BFN:10:3}
   HH=${BFN:13:2}
   MM=${BFN:15:2}
   SS=${BFN:17:2}
#
   DATESTRING="01Jan$YEAR $DAY days yesterday $HH:$MM:$SS"
   let OBSTIME=$(date -u +%s --date="$DATESTRING") >> $ERRLOG  2>&1  
   EXS=$?
   if [ $EXS != 0 ] ; then 
      echo "$REALTIME - $APPNAME - ERROR - Invalid date $DATESTRING"  >> $LOGFILE
      continue   # to next file
   fi  
   DATESTAMP=$(date -u +'%d%b%y'  --date="$DATESTRING" ) >> $ERRLOG  2>&1
   OBSDATE=$(date -u +'%Y %j %T %Z'  --date="$DATESTRING" ) >> $ERRLOG  2>&1 
#
#  Output and temp files
   TXTFILE=$DIR/$BFN".dat"
#   PWRIMG=$DIR/$BFN"_PWR.png"
#   SNRIMG=$DIR/$BFN"_SNR.png"
#   OIMG=$DIR/$BFN"_O.png"
#   XIMG=$DIR/$BFN"_X.png"

# Optionally remove the text file
   if (( $CLEAN )) ; then   
      echo "$REALTIME - $APPNAME - INFO - removing files $TXTFILE and $NCDFILE"  >> $LOGFILE
      rm -f $TXTFILE 
   fi
# 
# decompress as needed
   if [ -n "$BZIP2" ] ; then
      echo "$REALTIME - $APPNAME - INFO - bzip2 decompress $INFILE"  >> $LOGFILE
      bunzip2 -qkfd "$INFILE" >> $ERRLOG 2>&1 
      EXS=$? 
      if [ $EXS != 0 ] ; then 
         echo "$REALTIME - $APPNAME - ERROR - bunzip2 failed with error $EXS"  >> $LOGFILE
         continue # to next file
      fi  
   elif [ -n "$GZIP" ] ; then
      echo "$REALTIME - $APPNAME - INFO - gzip decompress $INFILE" >> $LOGFILE
#     gzip does not allow to keep the compressed file. Force this.
      cp -a "$INFILE" $TMPDIR/ 
      gunzip -qfd "$INFILE" >> $ERRLOG 2>&1
      if [ $EXS != 0 ] ; then 
         echo "$REALTIME - $APPNAME - ERROR - gunzip failed with error $EXS"  >> $LOGFILE
         continue # to next file 
      fi  
      mv "$TMPDIR/$(basename $INFILE)" "$(dirname $INFILE)"
   fi
#
   REALTIME=$(date -u +"$TFMT" )
   echo "$REALTIME - $APPNAME - INFO - INFILE=$INFILE RIQFILE=$RIQFILE OUTDIR=$OUTDIR TITLE=$TITLE"  >> $LOGFILE 
#
#  Determine site-specific variables 
   source 'station_settings.sh' 
#
#  Over-ride the noise level by command line
   if [ -n "$NOISE" ] ; then 
      RXNOISE="-n $NOISE"
   fi
#  Over-ride the receiver mask by command line
   if [ $MASK ] ; then
      RXMASK="-m $MASK"
      BMASK=$( echo  "obase=2;$MASK" | bc)
   else
      RXMASK="-m $SMASK"
      BMASK=$( echo  "obase=2;$SMASK" | bc)
   fi
#
#  pad to 8 places
   BMASK=$(printf "%.8d\n" $BMASK)
#
   PLOTFILE=$TMPDIR/plot.gnu
#
   REALTIME=$(date -u +"$TFMT" )
   echo "$REALTIME - $APPNAME - INFO - processing data from $RIQFILE"  >> $LOGFILE

#  Use the readriq program to extract the plot file from the
#  raw data file.  Many user-maintained options here!
#  This is a long process; do not repeat if already done
#  ToDo: Some form of auto-selection based on format
   if [ ! -s "$TXTFILE"  ] ; then
      if (( $UOR )) ; then 
#         riq_ionogram  -c -bf -U $URSI $RANGE0 $RXMASK $RXNOISE $NCDFLAG $RIQFILE > $TXTFILE
#        Old Wallops  data prior to 2009
         riq_ionogram-0 $RIQARGS -bf -U $URSI $RXMASK $RIQFILE > $TXTFILE
      else
         echo riq_ionogram  -c -bf  $RANGE0 $RXMASK $RXNOISE $NCDFLAG $RIQFILE
         riq_ionogram  $RIQARGS -bf $RANGE0 $RXMASK $RXNOISE $NCDFLAG $RIQFILE > $TXTFILE
      fi 
   fi 
#
   REALTIME=$(date -u +"$TFMT" )
   echo "$REALTIME - $APPNAME - INFO - plotting data from $TXTFILE"  >> $LOGFILE
#
   NRGT=$(dump_headers $RIQFILE | grep gate_count | awk '{print $3}')
   NFRQ=$(dump_headers $RIQFILE | grep base_steps | awk '{print $3}')
#  Loop over each frequency
   FRQLIST=$(seq 1 $NFRQ)
   for FRQ in $FRQLIST ; do
#     Pad the frequency counter to 4 places
      FRQ4=$(printf %04d $FRQ)
      OXIMG=$DIR/$BFN'_F'$FRQ4'.png'
#     Inform the calling program of the files created.
      echo $OXIMG  
#     Extract the frequency from the file
      NLINE=$(( (NRGT+2)*(FRQ-1) + 1 ))
      FREQUENCY=$(buthead $NLINE < $TXTFILE | head -1 | awk '{print $5}') 
#     Generate the gnuplot command file
   cat <<EOF > $PLOTFILE
set grid lt 22 lw 1
set xrange $RANGESPAN
set xtics $XTICS
set mxtics $MXTICS
set ytics 10
set mytics 10 
set yrange [0:$PWRMAX] 
set title "$STATION $OBSDATE       $DATESTAMP  $TITLE" font "arial,14"
set xlabel "Range [km]" font "arial,14"
set format x "%.1f"
set xtics font "arial,14"
set ylabel "Amplitide [dB]" font "arial,14"
set ytics font "arial,14"
set size ratio 0.5
set key top left
set label 1  "VIPIR  $BFN.RIQ" at screen 0.65,0.11 font "arial,10"
set label 2  "$FREQUENCY MHz"  at screen 0.85,0.82 font "arial,16"
set label 3  "RxMask $BMASK"   at screen 0.11,0.11 font "arial,10"
set term png font arial 12 size $SIZE  crop 
#
# ascan plot
set output "$OXIMG"
ASCAN=$FRQ-1
plot "$TXTFILE" using 6:9  every :1::ASCAN::ASCAN with points title "O" ,\
             "" using 6:11 every :1::ASCAN::ASCAN with points title "X"
set output
#
EOF
#
#
#     Plot the data
      gnuplot $PLOTFILE

   done  #   All ascan plots

#  Optionally remove the text file
   if (( $CLEAN )) ; then   
      echo "$REALTIME - $APPNAME - INFO - removing text file $TXTFILE"  >> $LOGFILE
      rm -f $TXTFILE
   fi

#  Remove original files if decompression was desired
   if [ -n "$BZIP2" ] || [ -n "$GZIP" ] ; then   
      echo "$REALTIME - $APPNAME - INFO - removing decompressed file $RIQFILE"  >> $LOGFILE
      rm -f $RIQFILE 2>>$ERRLOG
   fi

   REALTIME=$(date -u +"$TFMT" )
   echo "$REALTIME - $APPNAME - INFO - Finished processing $INFILE"  >> $LOGFILE

done   #  each file

exit 0