LOGICAL FUNCTION READPRI(IU,FLIPHASE,WORDFORMAT,   PRI)
!
! Read a single PRI from the integer Raw IQ file and return the complex I/Q values 
!
! Return value is TRUE if there is some problem, such as EOF
!
! 16 bit integers are in the file as high byte first (Big Endian)
! An offset value is applied to each range gate to prevent a value
! of precisely 0 from getting into log() functions
!
! This routine assumes that the SCT and PCT have already been read 
! and the relevant elements loaded into the PRI structure
!
! 01Jul12 -- Added FLIPHASE mask to rotate select phases by 180 degrees
!            this compensates for reversed cables or wrong pointing antennas. 
!
! 31May13 -- Added ability to read the 12+4 floating point format
!
! "To explore His Universe with Reverence"
!
!
IMPLICIT NONE
INCLUDE 'pct.f90'
INCLUDE 'pri.f90'
!
COMPLEX,PARAMETER :: CZero=(0.0, 0.0) 
INTEGER(KIND=2),PARAMETER :: MASK12=-16
INTEGER(KIND=2),PARAMETER :: MASK4=15 
INTEGER(KIND=1),PARAMETER :: MASK8=-16
!
INTEGER,INTENT(IN) :: IU,FLIPHASE
INTEGER(KIND=4),INTENT(IN)::WORDFORMAT
TYPE(PRItype),INTENT(INOUT) :: PRI
!
INTEGER:: IERR,RGT,IRX
INTEGER(KIND=4):: IPHAS4,IQUAD4
INTEGER(KIND=2):: IPHAS2,IQUAD2
INTEGER(KIND=1):: IPHASEXP,IQUADEXP
REAL(KIND=4):: RI,RQ
COMPLEX(KIND=4):: DEFAULT
!
!
! These equivalanced arrays are used to swap the bytes in the I/Q bytes
! because these i86 systems use little endian and the data are in big endian
! In theory the TRANSFER() intrinsic can be used to do this, but it will be slower
! and the intinisic is obtuse.
!
INTEGER(KIND=2),DIMENSION(2)::IPQ
INTEGER(KIND=1),DIMENSION(4)::IBP
EQUIVALENCE (IPQ(1),IBP(1))
!
!
! =============================================
!
!
IERR=0
READPRI=.FALSE.
PRI%PeakRangeGate=1
PRI%Noise=0.0
PRI%ASCAN=0.0
DEFAULT=(1.0,0.0)
IF (WORDFORMAT.EQ.2) DEFAULT=(0.1,0.0) 
!
! Get the Ascan data
DO RGT = 1, PRI%GateCount
   DO IRX = 1, PRI%ReceiverCount
      READ(IU,IOSTAT=IERR) IBP(2),IBP(1),IBP(4),IBP(3)
      IF (IERR.NE.0) THEN
         READPRI=.TRUE.
         RETURN
      ENDIF

      IF ((WORDFORMAT .EQ. 0) .OR. (WORDFORMAT.EQ.1)) THEN     ! 16 bit integers in Big Endian format
         IPHAS2=IPQ(1)
         IQUAD2=IPQ(2)
!int         IPHAS4=IPHAS2    ! Integer Approach
!int         IQUAD4=IQUAD2
         RI=REAL(IPHAS2)      ! Float Approach
         RQ=REAL(IQUAD2)
      ELSE IF (WORDFORMAT.EQ. 2) THEN               ! 12+4 floating point 
         IPHAS2=IPQ(1)                              ! 16 bit data word
         IQUAD2=IPQ(2)
!int         IPHAS4=ISHFT(IAND(IPHAS2,MASK12),4)       ! 12 bit mantissa 
!int         IQUAD4=ISHFT(IAND(IQUAD2,MASK12),4)
         IPHAS4=RSHIFT(IAND(IPHAS2,MASK12),4)       ! 12 bit mantissa 
         IQUAD4=RSHIFT(IAND(IQUAD2,MASK12),4)
         IPHASEXP=15-IAND(IPHAS2,MASK4)                ! 4 bit signed exponent.
         IQUADEXP=15-IAND(IQUAD2,MASK4)
!int         IPHAS4=RSHIFT(IPHAS4,-IPHASEXP)             ! Apply the exponent
!int         IQUAD4=RSHIFT(IQUAD4,-IQUADEXP) 
!db         write(*,*)IPHAS4,IPHASEXP,IQUAD4,IQUADEXP 
         RI=REAL(IPHAS4)*2.0**(IPHASEXP-3)        ! Apply the exponent, Float approach. The -3 is a 0 dB adjustment
         RQ=REAL(IQUAD4)*2.0**(IQUADEXP-3)        ! ToDo: Consider a shift approach
      ENDIF
!
!     Optionally flip phases
      IF (BTEST(FLIPHASE, IRX-1)) THEN
!int         PRI%Ascan(IRX,RGT)= CMPLX( -REAL(IPHAS4), -REAL(IQUAD4) ) ! Phase flip by 180 ; Integer
         PRI%Ascan(IRX,RGT)= CMPLX( -RI, -RQ ) ! Phase flip by 180 ; Float 
      ELSE
!int         PRI%Ascan(IRX,RGT)= CMPLX(  REAL(IPHAS4),  REAL(IQUAD4) ) ! Normal data
         PRI%Ascan(IRX,RGT)= CMPLX(  RI,  RQ ) ! Normal data
      ENDIF
!
      IF (PRI%Ascan(IRX,RGT).EQ.CZero) PRI%Ascan(IRX,RGT)= DEFAULT  
!      write(*,100) RGT,IRX,IPQ(1),IPQ(2),IPHAS2,IQUAD2,IPHAS4,IQUAD4,IPHAS4,IPHASEXP,IQUAD4,IQUADEXP,PRI%Ascan(IRX,RGT)
   ENDDO
ENDDO

RETURN
100 FORMAT (I6,1X,I3,1X,2I8,2Z8.4,2Z10.8,2(I10, I4),4F12.3)
END FUNCTION READPRI 
