PROGRAM riq_ionogram
! 
! Reader program for the Raw I/Q (RIQ) Version 1 data format
! for the Vertical Incidence Pulsed Ionospheric Radar (VIPIR) made by 
! Scion Associates.
!
! This code deals with the RIQ ionograms as images of received signal as
! a function of frequency and range.  Multiple derived computations on
! the received signals are generated by this code. 
!
! Author: Terence Bullett,   CIRES
! 20 September 2008
! Linux using gfortran compiler
!
!-----------------------------------------------------------------------------------------
!
! Open files. NOTE:  This code REQUIRES the equivalent of MODE=BINARY or ACCESS=STREAM.
!             This is generally a F2003 standard, or unique to commercial compilers.
!             G95 has this feature, gfortran does not.  Your mileage will vary 
!-----------------------------------------------------------------------------------------
!
! License:
! This software is the intellectual property of the author.
! Generous terms of use are likely if you ask nicely.
! 
! "To give glory to God by exploring His universe with reverence"  
!
! Revision History (incomplete)
!
! 29Oct08 TWB -- First version using 1.0 format RIQ data files
!                Adapted from Version 0.22
! 14Feb09 TWB -- Created Version 2.00.  BugFixes plus:
!                - Create netCDF files
!                - Convert SCT/PCT strings from C to FORTRAN
!                - Move the Ionogram data to a structure
!                - Range Gate Offset by Station and Date
!                - Rotated arrays for Jicamarca
!                - Parameters to bound range of amplitude statistics
! 14May09 TWB -- Version 2.03
!                - Added a noise threshold modification factor to set the noise
!                  level relative the Most Probable Amplitude. 
! 25Jan10 TWB -- Version 2.04
!                - Removed the FORALL() constructs, extremely slow
!                - Modifed to process "shuffle-mode" ionograms where every other
!                  pulse is a fixed frequency.
!                - Fixed bug which did not allow the command line RANGE0 value to be applied.
!                - Added symmetrical processing to the Fixed portion of shufflemode data
!                - Added time to the output, including range gate sample times.
! 01Oct10 TWB -- Version 2.05
!                - Added RTI mode processing
!                - Updated netCDF format to include more metadata
!                - Added meta-data over-ride from a config file
!                - Added stations SJJ18, AA109, PM91J
!                - Moved to gfortran 4.4.5, reads the binary files and cuts runtime in half 
! 14Mar11 TWB -- Version 2.06
!                - Change in phlisophy in shuffle mode data.  Now only one of the shuffled data sets are processed.
!                - Looking at multiple shuffled modes requires re-calling with different command switches
!
! 01Jul12 TWB -- Version 2.07
!                - Phase flip by command line, per antenna
!                - dP/dt phase shift "doppler" Todo"Update this to linear fit
!                - Peak mode to plot raw data at echo peaks, with rudimentary SNR thresholdiong
!                - Cable length phase adjustment 
!                - Simple Interferometry
!                - Updated control of statistics allowing negative values of range2 to exclude upper ranges
!
USE misc_utils
IMPLICIT NONE

INCLUDE 'pct.f90'   ! PCT
INCLUDE 'sct.f90'   ! SCT
INCLUDE 'pri.f90'   ! PRI
INCLUDE 'ionogram.f90' ! Processed ionogram structure
!
! Parameters
INTEGER, PARAMETER :: MAXPRI=2048
COMPLEX, PARAMETER :: CZERO=(0.0, 0.0)
REAL,PARAMETER :: VERSION=2.07
REAL,PARAMETER :: PI=4.0*ATAN(1.0)
REAL,PARAMETER :: DEGperRAD = 180.0/PI
!
! Local Variables
INTEGER :: I,J,NPRI,IU,IBLOCK,IFMX,IRMX,ISTACK,IRX,IASCAN,IRNG,NRGT,FixedFreq
INTEGER :: NFRQ,KFRQ,ISUM,IPRI,IRAMP,IPULSET,NPULSET,NPRIBLOCK,IFRQ,KPRI,MINRGT,MAXRGT,RGT1,RGT2
INTEGER :: NUMPRI,NUMSHUFFLE,ISHUFFLE,PICKSHUFFLE 
REAL    :: RANGE0,AMPMIN,AMPMAX,AMPMPA,RANGE1,RANGE2,TEMP,SNR,FREQ,UNDERFLOW
REAL,DIMENSION(8):: RI,RQ,RM,RP
REAL :: CTime
CHARACTER(LEN=120):: INFILE,NETCDFILE
CHARACTER(LEN=120) :: CLP
COMPLEX,DIMENSION(8,2) :: RxBeam ! Receiver Coefficients for the various beams.
LOGICAL FIRST
CHARACTER(LEN=5) :: URSICODE
CHARACTER(LEN=32):: TimeStamp,TimeHack
CHARACTER(LEN=128):: ConfigFile

!
! For command line parsing
INTEGER :: K,NA,N1,N2,NUMRANGE,ISELRX,IRGATE,CALMETHOD
LOGICAL :: EOF,SKIP,PEAK,COHERENT,VERBOSE,RAW,INFOK,BLANKA,BLANKP,BLANKF,STACKRAW
LOGICAL :: GATESELECT,SAMODE,SHUFFLEMODE,SKIPPRI,THRESH
LOGICAL :: IGRAM,NETCDF,DEBUG,RTIMODE,SIMODE,DPDTDOPP,RXCAL,PDFOUT
INTEGER :: MASKRX,FLIPHASE
REAL    :: FBANDL, FBANDH, NOISEMOD, THRESHOLD 
! 
!
! Structures
TYPE(PRItype) :: PRI(MAXPRI)
TYPE(SCTtype) :: SCT
TYPE(PCTtype) :: PCT 
TYPE(IonogramType) :: Ionogram
!
! Functions
LOGICAL :: READPRI, READSCT,READPCT
INTEGER:: POWERSUM
!
IU=1
EOF=.FALSE.
ISELRX=0
MASKRX=-1       ! Select all antenas by default
FLIPHASE=0     ! Assume all antennas are wired properly
RANGE0=-9999.0  ! A not-set value keyed on by InitIonogram()
N1=0
N2=0
NUMRANGE=-1
BLANKA=.FALSE.
BLANKP=.FALSE.
BLANKF=.FALSE.
VERBOSE=.FALSE.
PEAK=.FALSE.
RAW=.FALSE.
STACKRAW=.FALSE.
COHERENT=.FALSE.
GATESELECT=.FALSE.
SAMODE=.FALSE.
IGRAM=.FALSE.
NETCDF=.FALSE.
FIRST=.TRUE.
DEBUG=.FALSE.
SHUFFLEMODE=.FALSE.
THRESH=.FALSE.
NUMPRI=0
SKIPPRI=.FALSE.
NUMSHUFFLE=0
PICKSHUFFLE=1
RTIMODE=.FALSE.
SIMODE=.FALSE.
DPDTDOPP=.FALSE.
RXCAL=.FALSE.
PDFOUT=.FALSE.
CALMETHOD=0  ! Do not calibrate
URSICODE=""
FBANDL=0.0
FBANDH=999000.0
RM=0.0
RP=0.0
RI=0.0
RQ=0.0
IRGATE=0
RANGE1=0.0
RANGE2=0.0
NOISEMOD=1.0
FixedFreq=0.0
THRESHOLD=0.0
ConfigFile="Config.txt" 
KFRQ=0
IFRQ=0 
!
! Read the command line for arguments
K = IARGC() 
! Filename must exist
IF (K.LT.1) THEN
   WRITE(*,*) "read_riq-> ERROR: Input file must be specified"
   CALL EXIT(-1)
RETURN
ENDIF
!
SKIP=.FALSE.
COHERENT=.FALSE.
! Parse the command line.
DO NA = 1, K
   CALL GETARG(NA,CLP)
   IF (CLP.EQ.'-v') THEN
      VERBOSE = .TRUE.
   ELSE IF (CLP.EQ.'-h') THEN
      WRITE(*,*)' useage: riq_ionogram {} filename'
      WRITE(*,*)' '
      WRITE(*,'(A,F8.3)') 'Version: ',VERSION
      WRITE(*,*)'Decode VIPIR HF Radar Raw I/Q data file into ionogram image data'
      WRITE(*,*)' '
      WRITE(*,*)' -U URSICODE Set (over-ride) the station URSI code.'
      WRITE(*,*)' -r Raw plot of A-scans between N1 and N2. Ignores shuffle settings'
      WRITE(*,*)' -s Stacked raw plot of A-scans between N1 and N2.  Follows shuffle settings'
      WRITE(*,*)' -R RTI mode processing. Integrate over pulsets only'
      WRITE(*,*)' -S {n} Shuffle-mode data with {n} interleaves'
      WRITE(*,*)' -P {i} Pick the {i}th shuffled sequence to process.  i=1,2..n'  
      WRITE(*,*)' -G {n} Select only range gate n (raw modes)'
      WRITE(*,*)' -I Enable Simple Interferometry'
      WRITE(*,*)' -D Enable dP/dt Doppler calculations.  Forces -R RTI mode'
      WRITE(*,*)' -N {filename} Create netCDF file'
      WRITE(*,*)' -R0 {Range0} Set range gate offset'
      WRITE(*,*)' -n1 First Number'
      WRITE(*,*)' -n2 Second Number'
      WRITE(*,*)' -t {x} Set Threshold to {x}' 
      WRITE(*,*)' -cf {configuraton_file}  Read metadata from this file'
      WRITE(*,*)' ------------------'
      WRITE(*,*)' -p Peak Detect (for raw data)'
      WRITE(*,*)' -i Ionogram (incoherent)'
      WRITE(*,*)' -c Ionogram (coherent)'
      WRITE(*,*)' -d Debug info to stderr'
      WRITE(*,*)' -cal {method} ; Current methods are:'
      WRITE(*,*)'     0 - No calibration (default)'
      WRITE(*,*)'     1 - Simple cable length difference phase adjustment'
      WRITE(*,*)'         Others TBD'
      WRITE(*,*)' -g {n} Set # of range gates = n'
      WRITE(*,*)' -m {n} Set Rx Mask to n '
      WRITE(*,*)' -n {x} Set NoiseMod to x '
      WRITE(*,*)' -a {n} Select single receiver n'
      WRITE(*,*)' -fp {n} Mask for flipping phase by 180 degrees (mis-wired antennas)'
      WRITE(*,*)' -sa Spectrum Analyzer mode.  -c or -i needs to preceed!'
      WRITE(*,*)' -f1 Lower frequency select band [kHz]'
      WRITE(*,*)' -f2 Upper frequency select band [kHz]'
      WRITE(*,*)' -r1 Lower range for statistics (not output!) Default=30 [km]'
      WRITE(*,*)' -r2 Upper range for statistics (not output!) [km]'
      WRITE(*,*)'     Negative values are relative to the maximum range in the a-scan'
      WRITE(*,*)' -pdf output the signal amplitude probability distribution function'
      WRITE(*,*)' -ba Blank between each Antenna or A-scan'
      WRITE(*,*)' -bp Blank between eack PRI' 
      WRITE(*,*)' -bf Blank between each Frequency'
      WRITE(*,*)' -bi Blank between each Ionogram'
      WRITE(*,*)' -v Verbose'
      WRITE(*,*)' Filename must be provided on the command line.'
      WRITE(*,*)' Command line options:'
      WRITE(*,*)' Returns exit status as follows:'
      WRITE(*,*)'  0:  No Problems'
      WRITE(*,*)' -1:  Problems'
      RETURN
   ELSE IF (SKIP) THEN
      SKIP=.FALSE.
   ELSE IF (CLP.EQ.'-p') THEN
      PEAK=.TRUE.
   ELSE IF (CLP.EQ.'-N') THEN
      NETCDF=.TRUE.
      CALL GETARG(NA+1,NETCDFILE)
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-i') THEN
      COHERENT=.FALSE.
      IGRAM=.TRUE.
   ELSE IF (CLP.EQ.'-c') THEN
      COHERENT=.TRUE.
      IGRAM=.TRUE.
   ELSE IF (CLP.EQ.'-d') THEN
      DEBUG=.TRUE.
   ELSE IF (CLP.eq.'-cal') THEN
      RXCAL=.TRUE.
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) CALMETHOD
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-r') THEN
      RAW=.TRUE.
      IGRAM=.FALSE.
      STACKRAW=.FALSE.
      IF(N1.EQ.0) N1=1
      IF(N2.EQ.0) N2=1000
   ELSE IF (CLP.EQ.'-s') THEN
      STACKRAW=.TRUE.
      IGRAM=.FALSE.
      RAW=.FALSE.
      IF(N1.EQ.0) N1=1
      IF(N2.EQ.0) N2=1000
   ELSE IF (CLP.EQ.'-R') THEN
      RTIMODE=.TRUE.
   ELSE IF (CLP.EQ.'-S') THEN
      SHUFFLEMODE=.TRUE.
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) NUMSHUFFLE
      SKIP=.TRUE.
     ELSE IF (CLP.EQ.'-t') THEN
      THRESH=.TRUE.
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) THRESHOLD
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-P') THEN
      SHUFFLEMODE=.TRUE.
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) PICKSHUFFLE
      SKIP=.TRUE.     
   ELSE IF (CLP.EQ.'-G') THEN
      GATESELECT=.TRUE.
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) IRGATE
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-I') THEN
      SIMODE=.TRUE.
   ELSE IF (CLP.EQ.'-D') THEN
      DPDTDOPP=.TRUE.
      RTIMODE=.TRUE.
   ELSE IF (CLP.EQ.'-R0') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) RANGE0
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-a') THEN
!     Select a single antenna by setting MASKRX
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) ISELRX
      ISELRX=MIN(8,MAX(0,ISELRX))
      MASKRX=INT(2**(ISELRX-1))
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-n1') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) N1
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-n2') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) N2
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-g') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) NUMRANGE
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-m') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) MASKRX
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-fp') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) FLIPHASE
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-n') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) NOISEMOD
      SKIP=.TRUE.
   ELSE IF  (CLP.EQ.'-sa') THEN
      SAMODE=.TRUE.
      IGRAM=.FALSE.
   ELSE IF (CLP.EQ.'-f1') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) FBANDL
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-f2') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) FBANDH
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-r1') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) RANGE1
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-r2') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) RANGE2
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-U') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,'(A5)') URSICODE
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-N') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,'(A128)') ConfigFile
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-pdf') THEN
      PDFOUT=.TRUE.
   ELSE IF  (CLP.EQ.'-ba') THEN
      BLANKA=.TRUE.
   ELSE IF  (CLP.EQ.'-bp') THEN
      BLANKP=.TRUE.
   ELSE IF  (CLP.EQ.'-bf') THEN
      BLANKF=.TRUE.
   ELSE
!    This is the input file name
     INFILE=CLP
   ENDIF
ENDDO      
!
IF(DEBUG) THEN
   CALL FDATE(TimeHack)
   CALL CPU_TIME(CTime)
   WRITE(*,*) 'Start run',TimeHack,CTime 
ENDIF

!     Check if input file exists
INQUIRE(FILE=INFILE,EXIST=INFOK)
IF (.NOT.INFOK) THEN
   WRITE(*,*) ' readriq - error - Missing input file'
   CALL EXIT(-1)
ENDIF
!
! Zero out the ionogram (PRI) structures.
DO I=1,MAXPRI
   CALL ZeroPRI(PRI(I))
ENDDO
!
!
!-----------------------------------------------------------------------------------------
!
! Open files. NOTE:  This code REQUIRES the equivalent of MODE=BINARY or ACCESS=STREAM.
!             This is generally a F2003 standard, or unique to commercial compilers.
!             G95 has this feature, and in October 2010 gfortran version 4.4.5 also does.
!             Your mileage will vary. 
!-----------------------------------------------------------------------------------------

OPEN(UNIT=IU, FILE=INFILE, FORM="UNFORMATTED", ACCESS="STREAM")
! Read the SCT, PCT
EOF=READSCT(IU,  SCT)
IF (EOF) CALL EXIT(1)
CALL FixSCTstrings(SCT)
IF(VERBOSE) CALL DumpSCT(SCT)
!
! Initialize the Ionogram structure
! 
CALL InitIonogram(SCT,URSICODE,RANGE0,  Ionogram)
CALL ReadConfig(ConfigFile,  Ionogram)
!
RANGE0=Ionogram%RANGE0
!
! Set Range Gate Limits.
! Default Values
RGT1=15          ! To avoid Tx 
RGT2=Ionogram%GateCount
IF(RANGE1.GT.0) THEN
   RGT1=NINT((RANGE1-RANGE0)/(0.150*REAL(SCT%Timing%gate_step)))
   RGT1=MIN(MAX(1,RGT1),Ionogram%GateCount)
ENDIF
IF (RANGE2.GT.0) THEN
   RGT2=NINT((RANGE2-RANGE0)/(0.150*REAL(SCT%Timing%gate_step)))
   RGT2=MIN(MAX(1,RGT2),Ionogram%GateCount)
ELSE IF (RANGE2.LT.0) THEN
   RGT2=-NINT((RANGE2-RANGE0)/(0.150*REAL(SCT%Timing%gate_step)))
   RGT2=Ionogram%GateCount-RGT2
ENDIF
!
! Set amplitude limits
UNDERFLOW=1.0
IF (SCT%receiver%word_format .EQ.2) UNDERFLOW=0.001
!
! Update History 
CALL FDATE(TimeStamp)
WRITE(Ionogram%History,121) Version,TRIM(TimeStamp)
121 FORMAT ("Generated by riq_ionogram version ",F5.3, " at ",A) 
!
CALL SetRxBeams(Ionogram%URSICODE,Ionogram%EpochTime,  RxBeam)
!
! Set ShuffleMode based on the flags and SCT
SHUFFLEMODE= (SHUFFLEMODE.OR.(SCT%Frequency%tune_type.EQ.4).OR.(SCT%Frequency%tune_type.EQ.5))
IF ((SHUFFLEMODE).AND.(NUMSHUFFLE.EQ.0)) THEN
   NUMSHUFFLE = 2    ! A guess
ENDIF
! Don't let a default of 0 pass through 
IF (NUMSHUFFLE.EQ.0) NUMSHUFFLE = 1

!
! Ignore in raw dump mode.  Obey it in stacked mode.  Go figure.
IF (RAW) THEN
   SHUFFLEMODE=.FALSE.
   NUMSHUFFLE=1
   PICKSHUFFLE=1
ENDIF 

IF (DEBUG) WRITE(*,*) 'SHUFFLEMODE= ',SHUFFLEMODE,' NUMSHUFFLE= ',NUMSHUFFLE  
!
! Process this ionogram one B-Block at a time.  
!  V0 NPRIBLOCK=PCT%pulse_count*PCT%step_count*PCT%repeat_count
IF(RTIMODE) THEN
   NPRIBLOCK=SCT%frequency%pulse_count
ELSE
   NPRIBLOCK=SCT%frequency%pulse_count*SCT%frequency%ramp_steps*SCT%frequency%ramp_repeats
ENDIF
!
IF (DEBUG) WRITE(*,*) 'NPRIBLOCK= ',NPRIBLOCK
! 
IF(NPRIBLOCK.GT.MAXPRI) THEN
   WRITE(*,*) ' Block length ',NPRIBLOCK, ' greater than ',MAXPRI
   CALL EXIT (2)
ENDIF 
!
EOF=.FALSE.
NFRQ=0
IBLOCK=0
CTime=0.0
ISHUFFLE=0
IF(DEBUG) THEN
   CALL FDATE(TimeHack)
   CALL CPU_TIME(CTime)
   WRITE(*,*) 'Start Read Loop ',TimeHack, CTime
ENDIF

! Loop over all the ionograms in the data file
NUMPRI=0

! Only one header in PEAK mode
IF (PEAK)  WRITE(*,110) 'PRI','Rgt','Ascan','UT_Time','dt','Freq',('I',I,'Q',I,'M',I,'P',I,I=1,8)

DO WHILE ((NFRQ.LT.MAXFRQ).AND.(.NOT.EOF)) 
   IF(DEBUG) THEN
      CALL FDATE(TimeHack)
      CALL CPU_TIME(CTime)
      WRITE(*,*) 'Time= ',TimeHack, 'CPUTIME= ',CTime,'NFRQ= ',NFRQ
   ENDIF
   DO IPRI=1,NPRIBLOCK
      NPRI=NPRIBLOCK*IBLOCK + IPRI
      SKIPPRI=.TRUE.    ! Read at least one PRI
      DO WHILE (SKIPPRI.AND.(.NOT.EOF))
         NUMPRI=NUMPRI+1
         CALL ZeroPCT(PCT)
         EOF=READPCT(IU,PCT)
         IF(EOF) CYCLE
         CALL FixPCTstrings(PCT)
         IF(VERBOSE) CALL DumpPCT(PCT)
!        Copy relevant elements from SCT and PCT to the PRI Structure
         CALL SCTPCT2PRI(SCT,PCT,Ionogram%RANGE0, PRI(IPRI))
!        Over-ride number of range gates in the SCT by the command line
         IF(NUMRANGE.GT.0) PRI(IPRI)%GateCount=NUMRANGE
         EOF=READPRI(IU,FLIPHASE,SCT%receiver%word_format,   PRI(IPRI))         ! Read one A-scan
         IF(EOF) CYCLE
         ISHUFFLE = MOD((NUMPRI-1),NUMSHUFFLE)
         SKIPPRI=((SHUFFLEMODE).AND.(ISHUFFLE.NE.(PICKSHUFFLE-1)))
         IF (DEBUG) THEN
            WRITE(*,*) ' NUMPRI= ',NUMPRI, 'SKIPPRI= ',SKIPPRI,' ISHUFFLE= ',ISHUFFLE,' PICKSHUFFLE= ',PICKSHUFFLE
            WRITE(*,*) ' PRI%Frequency = ',PRI(IPRI) %Frequency
         ENDIF 
      ENDDO   ! Skipping PRI's 

!     Optional signal calibration
      IF (RXCAL) CALL CALIBRATE(Ionogram%URSICODE, Ionogram%Epochtime, CALMETHOD, MASKRX,   PRI(IPRI))
!
!     Compute peak and noise on a single PRI.
!     Print in 'stacked raw' mode.
!     Apply thresholding 
      IF (PEAK) THEN
         CALL PEAKPRI(MASKRX,  PRI(IPRI))
         IASCAN=0
         DO IRX=1,PRI(IPRI)%ReceiverCount
!           Select one range gate for all receivers or let the range vary from PRI to PRI?
!option            IRNG=PRI(IPRI)%PeakRangeGate(IRX)
            IRNG=PRI(IPRI)%PeakRangeGate(1)
            RI(IRX)=REAL( PRI(IPRI)%Ascan(IRX,IRNG))
            RQ(IRX)=AIMAG(PRI(IPRI)%Ascan(IRX,IRNG))
!           There should be no RI=RQ=0 data.
            RM(IRX)=10.*LOG10(RI(IRX)**2 + RQ(IRX)**2)
            RP(IRX)=DEGperRAD*ATAN2(RQ(IRX),RI(IRX))
         ENDDO       ! Receivers
         SNR= RM(1)-PRI(IPRI)%Noise(1)
         IF ((.NOT.THRESH).OR.(SNR.GT.THRESHOLD)) & 
         WRITE(*,111) NPRI,IRNG,IASCAN,PRI(IPRI)%Time, &
            (PRI(IPRI)%RelativeTime+PRI(IPRI)%GateStart+PRI(IPRI)%GateStep*(IRNG-1))/1.0E+9, &
            PRI(IPRI)%Frequency/1000.0,(NINT(RI(I)),NINT(RQ(I)),RM(I),RP(I),I=1,8)
         IF(BLANKA) WRITE(*,*)
         IF(BLANKP) WRITE(*,*)
      ENDIF  !  Peak Mode
         
      IF (RAW.AND.(NPRI.GE.N1).AND.(NPRI.LE.N2)) THEN
!        Output this raw data in column format
         WRITE(*,100) 'PRI','Rx','Rgt','Ascan','Stack','UT_Time','dt','Freq','I','Q','M','P'
         DO IRX=1,PRI(IPRI)%ReceiverCount
!           Determine if this receiver is masked out
            IF (.NOT.BTEST(MASKRX, IRX-1)) CYCLE
            IASCAN=PRI(IPRI)%ReceiverCount*(NPRI-1)+IRX
            DO IRNG=1,PRI(IPRI)%GateCount
               IF (GATESELECT.AND.(IRNG.NE.IRGATE)) CYCLE 
               ISTACK=PRI(IPRI)%GateCount*(IRX-1)+IRNG
               RI(1)=REAL( PRI(IPRI)%Ascan(IRX,IRNG))
               RQ(1)=AIMAG(PRI(IPRI)%Ascan(IRX,IRNG))
!              There should be no RI=RQ=0 data.
               RM(1)=10.*LOG10(RI(1)**2 + RQ(1)**2)
               RP(1)=DEGperRAD*ATAN2(RQ(1),RI(1))
               WRITE(*,101) NPRI,IRX,IRNG,IASCAN,ISTACK,PRI(IPRI)%Time, &
                           (PRI(IPRI)%RelativeTime+PRI(IPRI)%GateStart+PRI(IPRI)%GateStep*(IRNG-1))/1.0E+9, &
                            PRI(IPRI)%Frequency/1000.0, RI(1),RQ(1),RM(1),RP(1)
            ENDDO       ! Range Gates
            IF(BLANKA) WRITE(*,*)
         ENDDO          ! Receivers
         IF(BLANKP) WRITE(*,*)
      ENDIF             ! Raw 

      IF (STACKRAW.AND.(NPRI.GE.N1).AND.(NPRI.LE.N2)) THEN
!        Output this raw data in stacked format, all receivers on the same line
!        Does not obey receiver masking.
!
         WRITE(*,110) 'PRI','Rgt','Ascan','UT_Time','dt','Freq',('I',I,'Q',I,'M',I,'P',I,I=1,8)
         IASCAN=PRI(IPRI)%ReceiverCount*(NPRI-1)
         DO IRNG=1,PRI(IPRI)%GateCount
            IF (GATESELECT.AND.(IRNG.NE.IRGATE)) CYCLE
            RI=0.0
            RQ=0.0
            RM=0.0
            RP=0.0 
            DO IRX=1,PRI(IPRI)%ReceiverCount
               RI(IRX)=REAL( PRI(IPRI)%Ascan(IRX,IRNG))
               RQ(IRX)=AIMAG(PRI(IPRI)%Ascan(IRX,IRNG))
!              There should be no RI=RQ=0 data.
               RM(IRX)=10.*LOG10(RI(IRX)**2 + RQ(IRX)**2)
               RP(IRX)=DEGperRAD*ATAN2(RQ(IRX),RI(IRX))
            ENDDO       ! Receivers
            WRITE(*,111) NPRI,IRNG,IASCAN,PRI(IPRI)%Time, &
               (PRI(IPRI)%RelativeTime+PRI(IPRI)%GateStart+PRI(IPRI)%GateStep*(IRNG-1))/1.0E+9, &
               PRI(IPRI)%Frequency/1000.0,(RI(I),RQ(I),RM(I),RP(I),I=1,8)
         ENDDO       ! Range Gates
         IF(BLANKA) WRITE(*,*)
         IF(BLANKP) WRITE(*,*)
      ENDIF             ! Raw 
   ENDDO       ! One B-block

100 FORMAT ('#',A5,A4,3A7,A12,A14,A12,8(2A13,2A10))
101 FORMAT (I6,I4,3I7,F12.3,F14.7,F12.6,4(1X,F12.3,1X,F12.3,1X,F9.3,1X,F9.3))
110 FORMAT ('#',A5,2A7,A12,A14,A12,8(2(11X,A1,I1),2(8X,A1,I1)))
111 FORMAT (I6,2I7,F12.3,F14.7,F12.6,8(1X,F12.3,1X,F12.3,1X,F9.3,1X,F9.3))
         

   IF(EOF) CYCLE
   IBLOCK=IBLOCK+1
!
!  Don't proceed if just raw printing
   IF(RAW.OR.STACKRAW) CYCLE
!
!
   IF (DEBUG) THEN
      WRITE(*,*)  "SCT%frequency%ramp_steps  SCT%frequency%pulse_count  PCT%repeat_id PCT%pulse_id"
      WRITE(*,*)  SCT%frequency%ramp_steps, SCT%frequency%pulse_count, PCT%repeat_id, PCT%pulse_id
   ENDIF

   NPULSET =  SCT%frequency%pulse_count
!
!  Simple Interferometry
   IF (SIMODE) THEN
      DO IPULSET = 1, NPULSET    ! Each Pulset
         CALL SIMPLE_INTERFEROMETER(PRI(IPULSET))
      ENDDO
   ENDIF 

!  01Aug10 New feature: RTI mode processing.
!  This averages over only the pulset {SCT%frequency%pulse_count}.
   IF (RTIMODE) THEN
      IPRI=1
!
      NRGT=PRI(IPRI)%GateCount              ! For convenient reference
!xx      IFMX=SCT%frequency%pulse_count        ! The Frequency index multiplier
!xx      IRMX=IFMX* SCT%frequency%ramp_steps   ! The Ramp index multiplier
!     NFRQ are the number of previously processed base freqs.  KFRQ is the the one currently being integrated.
      KFRQ = NFRQ + 1
      IF(DEBUG) WRITE(*,*) 'KFRQ= ',KFRQ,' NFRQ= ', NFRQ,' IFRQ= ',IFRQ 
!     Zero the sums.  FORALL tends to be inefficient
      DO J=1,NRGT
         Ionogram%TPower(J,KFRQ)= 0.0
         Ionogram%OPower(J,KFRQ)= 0.0
         Ionogram%XPower(J,KFRQ)= 0.0
      ENDDO
      ISUM=0
!     Select all PRI's in the pulset
      DO IPULSET = 1, NPULSET   ! Each Pulset
!xx               KPRI=IRMX*(IRAMP-1) + IFMX*(IFRQ-1) + IPULSET
         KPRI=IPULSET
         Ionogram%Frequency(KFRQ)=PRI(KPRI)%Frequency
         ISUM = ISUM + PowerSum(COHERENT,MASKRX,KFRQ,PRI(KPRI),RxBeam,  Ionogram)
         IF (DEBUG) THEN
            CALL CPU_TIME(CTime)
            WRITE(*,*) "Time = ",CTime
            WRITE(*,*) "KFRQ,IFRQ,IRAMP,IPULSET,ISUM,KPRI,PRI(KPRI)%Frequency"
            WRITE(*,*) KFRQ,IFRQ,IRAMP,IPULSET,ISUM,KPRI,PRI(KPRI)%Frequency
         ENDIF 
      ENDDO ! Pulset
!     Finished adding up all values in the pulset at this nominal frequency.
!     Average values Disallow values < UNDERFLOW, which is data word format dependent   
      Ionogram%NumAve(KFRQ)=ISUM
      TEMP=REAL(ISUM)
!     a FORALL construct here is very inefficient 
      DO J=1,NRGT
         Ionogram%TPower(J,KFRQ)= MAX(Ionogram%TPower(J,KFRQ)/TEMP, UNDERFLOW)
         Ionogram%OPower(J,KFRQ)= MAX(Ionogram%OPower(J,KFRQ)/TEMP, UNDERFLOW)
         Ionogram%XPower(J,KFRQ)= MAX(Ionogram%XPower(J,KFRQ)/TEMP, UNDERFLOW)
      ENDDO 
!
!     Compute the average phase change in Hz 
      IF (DPDTDOPP) THEN 
         CALL DPDT(NPULSET,MASKRX,KFRQ,PRI,   Ionogram)
      ENDIF 
! 
!     Approximate UT time.  ToDo:  Average the time of the PRI's.
      Ionogram%Time(KFRQ)   = PRI(NPRIBLOCK/2+1)%Time
      NFRQ=NFRQ+SCT%frequency%ramp_steps
!     END OF RTI mode processing
!
   ELSE
!     Start averaging the PRI's into each ionogam frequency-range array
!     This partly un-wraps the B-mode ramps by looping over frequency, then ramps
!
      IPRI=1
!
      NRGT=PRI(IPRI)%GateCount              ! For convenient reference
      IFMX=SCT%frequency%pulse_count        ! The Frequency index multiplier
      IRMX=IFMX* SCT%frequency%ramp_steps   ! The Ramp index multiplier  (16Feb09 TWB Changed from ramp_repeats)
      DO IFRQ=1, SCT%frequency%ramp_steps   ! Each B-ramp
!        NFRQ are the number of previously processed base freqs.  KFRQ is the the one currently being integrated.
         KFRQ = NFRQ + IFRQ
         IF(DEBUG) WRITE(*,*) 'KFRQ= ',KFRQ,' NFRQ= ', NFRQ,' IFRQ= ',IFRQ 
!        Zero the sums.  FORALL tends to be inefficient
         DO J=1,NRGT
            Ionogram%TPower(J,KFRQ)= 0.0
            Ionogram%OPower(J,KFRQ)= 0.0
            Ionogram%XPower(J,KFRQ)= 0.0
         ENDDO
         ISUM=0
!        Select all PRI's with the same base frequency
         DO IRAMP=1, SCT%frequency%ramp_repeats          ! One B-Block
            DO IPULSET = 1, SCT%frequency%pulse_count    ! Each Pulset
               KPRI=IRMX*(IRAMP-1) + IFMX*(IFRQ-1) + IPULSET           
               Ionogram%Frequency(KFRQ)=PRI(KPRI)%Frequency
               ISUM = ISUM + PowerSum(COHERENT,MASKRX,KFRQ,PRI(KPRI),RxBeam,  Ionogram)
               IF (DEBUG) THEN
                  CALL CPU_TIME(CTime)
                  WRITE(*,*) "Time = ",CTime
                  WRITE(*,*) "KFRQ,IFRQ,IRAMP,IPULSET,ISUM,KPRI,PRI(KPRI)%Frequency"
                  WRITE(*,*) KFRQ,IFRQ,IRAMP,IPULSET,ISUM,KPRI,PRI(KPRI)%Frequency
               ENDIF 
            ENDDO ! Pulset
         ENDDO    ! Ramps
!        Finished adding up all values in the pulset at this nominal frequency.
!        Average values Disallow values < UNDERFLOW, which is data word format dependent  
         Ionogram%NumAve(KFRQ)=ISUM
!
         TEMP=REAL(ISUM)
!        a FORALL construct here is very inefficient 
         DO J=1,NRGT
            Ionogram%TPower(J,KFRQ)= MAX(Ionogram%TPower(J,KFRQ)/TEMP, UNDERFLOW)
            Ionogram%OPower(J,KFRQ)= MAX(Ionogram%OPower(J,KFRQ)/TEMP, UNDERFLOW)
            Ionogram%XPower(J,KFRQ)= MAX(Ionogram%XPower(J,KFRQ)/TEMP, UNDERFLOW)
         ENDDO 
!
!        Approximate UT time.  ToDo:  Average the time of the PRI's.
         Ionogram%Time(KFRQ)   = PRI(NPRIBLOCK/2+1)%Time
      ENDDO      ! This block 
      NFRQ=NFRQ+SCT%frequency%ramp_steps
!     Update the number of frequencies in the ionogram.
      Ionogram%NFREQ=MAX(Ionogram%NFREQ, NFRQ)
!     End of normal B-block processing
   ENDIF       
!
ENDDO          ! Each Ionogram in the file
   
NPRI=NPRI-1

IF(DEBUG) THEN
   CALL FDATE(TimeHack)
   WRITE(*,*) 'End Read Loop ',TimeHack 
   WRITE(*,*) '# NPRI= ',NPRI,' NFRQ= ',NFRQ
ENDIF
!
! Convert ionogram to dB
! Compute Noise values
!
NRGT=PRI(1)%GateCount
DO IFRQ=1,NFRQ
   IF (DEBUG) WRITE(*,99) (IFRQ,IRNG,Ionogram%TPower(IRNG,IFRQ),Ionogram%OPower(IRNG,IFRQ),Ionogram%XPower(IRNG,IFRQ), IRNG=1,NRGT)

   DO IRNG=1,NRGT 
      Ionogram%TPower(IRNG,IFRQ)= 10.0*LOG10(Ionogram%TPower(IRNG,IFRQ))
      Ionogram%OPower(IRNG,IFRQ)= 10.0*LOG10(Ionogram%OPower(IRNG,IFRQ))
      Ionogram%XPower(IRNG,IFRQ)= 10.0*LOG10(Ionogram%XPower(IRNG,IFRQ))
   ENDDO
99 FORMAT (2I4,3F12.2)
!  
!  Compute Total Power statistics
   FREQ=Ionogram%Frequency(IFRQ)
   CALL MPANOISE(RGT1,RGT2,NRGT,NOISEMOD,PDFOUT,FREQ,Ionogram%TPower(1:NRGT,IFRQ), AMPMIN,MINRGT,AMPMAX,MAXRGT,AMPMPA)
!   write(*,*) RGT1,RGT2,NRGT,NOISEMOD,Ionogram%TPower(1:NRGT,IFRQ)
!  write(*,*) IFRQ, AMPMIN,MINRGT,AMPMAX,MAXRGT,AMPMPA 
   Ionogram%TNoiseLevel(IFRQ)= AMPMPA
   Ionogram%TPeakLevel(IFRQ) = AMPMAX
   Ionogram%TPeakRange(IFRQ) = Ionogram%Range(MAXRGT)
   Ionogram%TMinLevel(IFRQ)  = AMPMIN
   Ionogram%TMinRange(IFRQ)  = Ionogram%Range(MINRGT)
!  Compute Ordianry Power statistics
   CALL MPANOISE(RGT1,RGT2,NRGT,NOISEMOD,.FALSE.,FREQ,Ionogram%OPower(1:NRGT,IFRQ),  AMPMIN,MINRGT,AMPMAX,MAXRGT,AMPMPA)
   Ionogram%ONoiseLevel(IFRQ)= AMPMPA
   Ionogram%OPeakLevel(IFRQ) = AMPMAX
   Ionogram%OPeakRange(IFRQ) = Ionogram%Range(MAXRGT)
   Ionogram%OMinLevel(IFRQ)  = AMPMIN
   Ionogram%OMinRange(IFRQ)  = Ionogram%Range(MINRGT)
!  Compute eXtraordinary Power statistics
   CALL MPANOISE(RGT1,RGT2,NRGT,NOISEMOD,.FALSE.,FREQ,Ionogram%XPower(1:NRGT,IFRQ),  AMPMIN,MINRGT,AMPMAX,MAXRGT,AMPMPA)
   Ionogram%XNoiseLevel(IFRQ)= AMPMPA
   Ionogram%XPeakLevel(IFRQ) = AMPMAX
   Ionogram%XPeakRange(IFRQ) = Ionogram%Range(MAXRGT)
   Ionogram%XMinLevel(IFRQ)  = AMPMIN
   Ionogram%XMinRange(IFRQ)  = Ionogram%Range(MINRGT)
ENDDO
!
! Write the NetCDF file, if wanted
IF(DEBUG) THEN
   CALL FDATE(TimeHack)
   WRITE(*,*) 'Start NetCDF write ',TimeHack 
ENDIF
IF (NETCDF) CALL Write_netCDF3(NETCDFILE, Ionogram)
!
! Output the ionogram data
IF(DEBUG) THEN
   CALL FDATE(TimeHack)
   WRITE(*,*) 'Start Ionogram Output ',TimeHack 
ENDIF
!
IF (IGRAM) THEN
   DO IFRQ=1, NFRQ
      WRITE(*,102) 'IFRQ','IRNG','UT_Time','dt','Freq','Range','TPower','TSNR','OPower','OSNR','XPower','XSNR','DPDT','DPDTv'
      IF( (Ionogram%Frequency(IFRQ).LT.FBANDL).OR.(Ionogram%Frequency(IFRQ).GT.FBANDH) ) CYCLE
      DO IRNG=1,PRI(1)%GateCount
         WRITE(*,103) IFRQ, IRNG, Ionogram%Time(IFRQ), Ionogram%Time(IFRQ)-Ionogram%Time(1),&
         Ionogram%Frequency(IFRQ)/1000.0, Ionogram%Range(IRNG), & 
         Ionogram%TPower(IRNG,IFRQ), Ionogram%TPower(IRNG,IFRQ)-Ionogram%TNoiseLevel(IFRQ), &
         Ionogram%OPower(IRNG,IFRQ), Ionogram%OPower(IRNG,IFRQ)-Ionogram%ONoiseLevel(IFRQ), &
         Ionogram%XPower(IRNG,IFRQ), Ionogram%XPower(IRNG,IFRQ)-Ionogram%XNoiseLevel(IFRQ), &
         Ionogram%DPDT(IRNG,IFRQ), SQRT(Ionogram%DPDT_Var(IRNG,IFRQ)) 
      ENDDO
      IF(BLANKF) WRITE(*,*)
   ENDDO
ELSE IF (SAMODE) THEN
   WRITE(*,104) 'Time','Freq', 'Rx','TMin','TMinRng','TNoise','TPeak','TPeakRng', &
                                    'OMin','OMinRng','ONoise','OPeak','OPeakRng', &
                                    'XMin','XMinRng','XNoise','XPeak','XPeakRng' 
   DO IFRQ=1, NFRQ
      IF( (Ionogram%Frequency(IFRQ).LT.FBANDL).OR.(Ionogram%Frequency(IFRQ).GT.FBANDH) ) CYCLE
      WRITE(*,105) Ionogram%Time(IFRQ),Ionogram%Frequency(IFRQ)/1000.0,ISELRX, &
       Ionogram%TMinLevel(IFRQ),Ionogram%TMinRange(IFRQ),Ionogram%TNoiseLevel(IFRQ), &
                                           Ionogram%TPeakLevel(IFRQ),Ionogram%TPeakRange(IFRQ), &
       Ionogram%OMinLevel(IFRQ),Ionogram%OMinRange(IFRQ),Ionogram%ONoiseLevel(IFRQ), &
                                           Ionogram%OPeakLevel(IFRQ),Ionogram%OPeakRange(IFRQ), &
       Ionogram%XMinLevel(IFRQ),Ionogram%XMinRange(IFRQ),Ionogram%XNoiseLevel(IFRQ), &
                                           Ionogram%XPeakLevel(IFRQ),Ionogram%XPeakRange(IFRQ)
      IF(BLANKF) WRITE(*,*)
   ENDDO
   WRITE(*,*) 
ENDIF
   
IF(DEBUG) THEN
   CALL FDATE(TimeHack)
   WRITE(*,*) 'End Run ',TimeHack 
ENDIF

102 FORMAT ('#',A7,A8,1X,4A12,A8,8A8)
103 FORMAT (2I8,1X,4F12.5, F8.2, 8F8.2)
104 FORMAT ('#',A11,A12,A4,3(5A9))
105 FORMAT (2F12.5,I4,3(5F9.2))

END 

