SUBROUTINE SetRxBeams(URSIcode,EpochTime,  RxBeam)
!
! 01Oct08   Terry Bullett
!
! Determine the complex receiver beamforming multipliers for digital beamforming.
! Currently this is all hard-wired because antenna information is not present in Version 0 RIQ files.
! and the beams are limited to overhead O and X.
!
! 02Mar09 TWB -- Change interface to include EpochTime, to include hard-coded changes to the receive 
!                antenna field which impact this calculation
!
! RxBeam(8,2) -- Array is a complex multiplier for digital beamforming for 8 antennas and 2 beams.
!                The 2 beams defined are O and X, although more are possible.  This is site specific
!                depending on the array geometry and the nature of the amplitude and phase of the O&X
!                waves
!
! ToDo: Pass the SCT (or Ionogram) structure and actually use the direction values in the data files
! 
!  18Nov13  TWB  Added Tupiza TZJ2J
! 
USE misc_utils 

! Dummy variables
CHARACTER(LEN=5),INTENT(IN) :: URSIcode
INTEGER(KIND=4) :: EpochTime
COMPLEX,DIMENSION(8,2),INTENT(OUT) :: RxBeam ! Receiver Coefficients for the various beams.

! Local Variables
INTEGER IRX
COMPLEX TEMP
COMPLEX,PARAMETER :: One=(1.0, 0.0)
COMPLEX,PARAMETER :: MinusOne=(-1.0, 0.0)
COMPLEX,PARAMETER :: Zero=(0.0, 0.0)
COMPLEX,PARAMETER :: Plus90=(0.0, 1.0)
COMPLEX,PARAMETER :: Minus90=(0.0, -1.0)
!
! Wallops:  Has a symmetric cross with odd numbered antennas in the N/S (+y) direction and even numbers
! Boulder:  directed E/W  (+x) direction.  Phase shift the +y antennas by +/- 90 degrees and add to the
!           +x direction
!
! Jicmarca: Initial installation has odd antennas in the +y magnetic N/S direction for O polarization
!           and even numbers in the magnetic E/W (+x) direction for X polarization.  Odd antennas need 
!           to be selected for X and even numbers selected for O.  Antennas 1 and 2 are swapped for 
!           data a the time of this writing.
! Tupiza:  Odd numberd antennas are in the +x 
!
!
SELECT CASE (URSIcode)

CASE ('XX000')   ! The default case assumes identical to Wallops.

   DO IRX=1,8
      IF(ODD(IRX)) THEN
         RxBeam(IRX,1)=One
         RxBeam(IRX,2)=One
      ELSE
         RxBeam(IRX,1)=Minus90
         RxBeam(IRX,2)=Plus90
      ENDIF
   ENDDO

CASE ('WI937')   ! Wallops Island  After 17Jun09, Wallops VIPIR upgraded to RIQ Format v1.2+

   DO IRX=1,8
      IF(ODD(IRX)) THEN
         RxBeam(IRX,1)=One
         RxBeam(IRX,2)=One
      ELSE
         RxBeam(IRX,1)=Minus90
         RxBeam(IRX,2)=Plus90
      ENDIF
   ENDDO

CASE('AA109')  ! Addis Ababa: Magnetic North is in the +x,-y direction, or -45 deg in antenna coordinates
   DO IRX=1,8
      IF(ODD(IRX)) THEN
         RxBeam(IRX,1)=One
         RxBeam(IRX,2)=One
      ELSE
         RxBeam(IRX,1)=MinusOne
         RxBeam(IRX,2)=One
      ENDIF
   ENDDO 

CASE('JM91J')                           ! Jicamarca  O mode is N/S linear, X is E/W linear
   IF (EpochTime.GT.1231398000) THEN   !  08 Jan 2009 when antenna were rotated 45 degrees 
      DO IRX=1,8
         IF(ODD(IRX)) THEN
            RxBeam(IRX,1)=One
            RxBeam(IRX,2)=One
         ELSE
            RxBeam(IRX,1)=MinusOne
            RxBeam(IRX,2)=One
         ENDIF
      ENDDO 
!     Antenna #5 is rotated in the opposite direction as the others.
!     This was corrected 10 March 2009
      IF (EpochTime.LT. 1236664800) THEN 
         RxBeam(5,1) = RxBeam(5,1)*MinusOne     
         RxBeam(5,2) = RxBeam(5,2)*MinusOne
      ENDIF
   ELSE
!     Antennas aligned N/S and E/W
      DO IRX=1,8
         IF(ODD(IRX)) THEN
            RxBeam(IRX,1)=One
            RxBeam(IRX,2)=Zero
         ELSE
            RxBeam(IRX,1)=Zero
            RxBeam(IRX,2)=One
         ENDIF
      ENDDO
      IF (EpochTime.LT.1227164400) THEN  !  20Nov08  is a guess
!        Swap Rx1 and Rx2
         TEMP=RxBeam(1,1)
         RxBeam(1,1)=RxBeam(2,1)
         RxBeam(2,1)=TEMP
         TEMP=RxBeam(1,2)
         RxBeam(1,2)=RxBeam(2,2)
         RxBeam(2,2)=TEMP
      ENDIF         ! Swap 1 & 2 
   ENDIF

CASE('BD840')    ! Boulder 
   DO IRX=1,8
      IF(ODD(IRX)) THEN
         RxBeam(IRX,1)=One
         RxBeam(IRX,2)=One
      ELSE
         RxBeam(IRX,1)=Minus90
         RxBeam(IRX,2)=Plus90
      ENDIF
   ENDDO

CASE('SJJ18')    ! San Juan has odd antennas pointed +x (East) and even antennas pointed +y (North) 
   DO IRX=1,8
      IF(ODD(IRX)) THEN
         RxBeam(IRX,1)=Minus90
         RxBeam(IRX,2)=Plus90
      ELSE
         RxBeam(IRX,1)=One
         RxBeam(IRX,2)=One
      ENDIF
   ENDDO

CASE ('KI100')   ! Kisumu is in the southern magnetic hemisphere with odd antennas in the +x and even in the +y

   DO IRX=1,8
      IF(ODD(IRX)) THEN
         RxBeam(IRX,1)=One
         RxBeam(IRX,2)=One
      ELSE
         RxBeam(IRX,1)=Plus90
         RxBeam(IRX,2)=Minus90
      ENDIF
   ENDDO

CASE ('TZJ2J')   ! Tupiza is in the southern magnetic hemisphere with odd antennas in the +x and even in the +y

   DO IRX=1,8
      IF(ODD(IRX)) THEN
         RxBeam(IRX,1)=One
         RxBeam(IRX,2)=One
      ELSE
         RxBeam(IRX,1)=Plus90
         RxBeam(IRX,2)=Minus90
      ENDIF
   ENDDO

CASE DEFAULT     ! Unknown Configuration Condition, set the beam pattern to mid-latitude northern hemisphere (e.g. San Juan) 

   DO IRX=1,8
      IF(ODD(IRX)) THEN
         RxBeam(IRX,1)=Minus90
         RxBeam(IRX,2)=Plus90
      ELSE
         RxBeam(IRX,1)=One
         RxBeam(IRX,2)=One
      ENDIF
   ENDDO

END SELECT

RETURN
END
