#!/bin/bash
# spectrumplot-3D.sh
# makes a 3D plot of Spectrum Analyzer Mode (.sam) data
# files from spectrumplot.sh  
#
# Terry Bullett
# University of Colorado
#
# Version 2.06  22Feb11
# useage:  spectrumplot-3D.sh datafile
# 26May11  -- Added -x EXTRA output filename feature
#          -- Added -L log frequency scale option 
#
# Requires:
#    msttcorefonts (included)
#    bc
#    station_settings.sh
#
#
export GDFONTPATH=/usr/share/fonts/truetype/msttcorefonts/
#export GDFONTPATH=$HOME/lib/msttcorefonts/
#
#
APPNAME=spectrumplot-3D
TFMT='%F %T %Z'
REALTIME=$(date -u +"$TFMT" )
YYYY=$(date -u +%Y)
DDD=$(date -u +%j)
LOGFILE="$HOME/log/$APPNAME-$YYYY-$DDD.log"
ERRLOG="$HOME/log/$APPNAME-$YYYY-$DDD.errlog"
TMPDIR="$HOME/tmp/$APPNAME"
mkdir -p $TMPDIR
declare -i LINEAR
#
# User-adustable plot parameters
SIZE="1024,768"
FRQSPAN="[1.0:25.0]"
LINEAR=$((1))
PWRMIN=0
PWRMAX=90
SNRMAX=50
OXMAX=50
SIZE="1024,768"
EXTRA=""
#
while getopts 'f:hLt:x:' OPTION
do
   case $OPTION in
      f) FRQSPAN="$OPTARG"
         ;;
      h)
         echo "spectrumplot-3D Version 2.06"
	 echo "Plot VIPIR spectrum analyzer mode data in 3D"
         echo "usage:  spectrumplot-3D.sh {options} SAMFILES"
         echo "SAMFILES are the merged text file outputs of spectrumplot.sh"
         echo ""
         echo "Optional switches"
         echo "-f FRQSPAN Change the frequency span of the plot, default is $FRQSPAN"
         echo "-L set log scale to frequency plot"
	 echo "-x EXTRA  Adds EXTRA string to the output files"
	 echo ""
	 exit 0 
	 ;;
      L) LINEAR=$((0))
         ;; 
      t) TITLE="$OPTARG"
         ;;
      x) EXTRA="$OPTARG"
	 ;;
   esac
done
#
# Frequency axis scaling
   YTICS='(1.5,2,3,4,5,7,10,15,20)'
   MYICS=''
   FSCALE='set logscale y'
   if (( $LINEAR )) ; then
      YTICS='2'
      FSCALE='unset logscale y'
      MYTICS='10'
   fi
#
# Remove the switches from the list.
shift $((OPTIND - 1))
#
# Process the input file list
FILELIST="$@"
for INFILE in $FILELIST ; do 
   REALTIME=$(date -u +"$TFMT" )
   if [ -z $INFILE ] ; then 
      echo "$REALTIME - $APPNAME - FATAL - Input file is undefined" >> $LOGFILE 
      exit -1
   fi
   if [ ! -r $INFILE ] ; then 
      echo "$REALTIME - $APPNAME - FATAL - Input file $INFILE does not exist or is non-readable" >> $LOGFILE 
      exit -2
   fi
#
   if [ -z $OUTDIR ] ; then
      DIR=$(dirname $INFILE)
   else
      DIR=$OUTDIR
   fi

#
#  Determine if the data file is compressed.
#
#
#  Determine if the data file is compressed.
#  ZIP compression is not supported, but it could be.
#  need to preserve the ZIP file.
#
   BZIP2=$(file $INFILE | grep bzip2)
   GZIP=$(file $INFILE | grep gzip)
   
#
#  Determine file names
   if [ -n "$BZIP2" ] ; then 
         DATFILE="$(dirname $INFILE)/$(basename $INFILE .bz2)" 
   elif [ -n "$GZIP" ] ; then
      DATFILE="$(dirname $INFILE)/$(basename $INFILE .gz)"
   else
      DATFILE="$INFILE"
   fi 
   BFN=$(basename $DATFILE .sam)
#  For v0 data files with no station ID in the filename
   if (( $UOR )) ; then
      URSI="$URSI"
   else 
      URSI=${BFN:0:5}
   fi 

#  Determine the measurement time from the standard filename
#  This should be a dayfile.
   YEAR=${BFN:6:4}
   DAY=${BFN:10:3}
   HH=00
   MM=00
   SS=00
#   HH=${BFN:13:2}
#   MM=${BFN:15:2}
#   SS=${BFN:17:2}
#
   DATESTRING="01Jan$YEAR $DAY days yesterday $HH:$MM:$SS"
   let OBSTIME=$(date -u +%s --date="$DATESTRING") >> $ERRLOG  2>&1  
   EXS=$?
   if [ $EXS != 0 ] ; then 
      echo "$REALTIME - $APPNAME - ERROR - Invalid date $DATESTRING"  >> $LOGFILE
      continue   # to next file
   fi  
   DATESTAMP=$(date -u +'%d%b%y'  --date="$DATESTRING" ) >> $ERRLOG  2>&1
   OBSDATE=$(date -u +'%Y %j'  --date="$DATESTRING" ) >> $ERRLOG  2>&1 
#
#  Output and temp files
   PLOTFILE=$TMPDIR/plot.gnu
   TXTFILE="$DIR/$BFN$EXTRA"'.sam'
   MAXIMG="$DIR/$BFN$EXTRA"'_saMAX.png'
   MINIMG="$DIR/$BFN$EXTRA"'_saMIN.png'
   MPAIMG="$DIR/$BFN$EXTRA"'_saMPA.png'
   SNRIMG="$DIR/$BFN$EXTRA"'_saSNR.png'

   REALTIME=$(date -u +"$TFMT" )
   echo "$REALTIME - $APPNAME - INFO - INFILE=$INFILE OUTDIR=$DIR TITLE=$TITLE"  >> $LOGFILE 
#
#  Determine site-specific variables 
   source 'station_settings.sh'
# 
# decompress as needed
   if [ -n "$BZIP2" ] ; then
      echo "$REALTIME - $APPNAME - INFO - bzip2 decompress $INFILE"  >> $LOGFILE
      bunzip2 -qkfd "$INFILE" >> $ERRLOG 2>&1 
      EXS=$? 
      if [ $EXS != 0 ] ; then 
         echo "$REALTIME - $APPNAME - ERROR - bunzip2 failed with error $EXS"  >> $LOGFILE
         continue # to next file
      fi  
   elif [ -n "$GZIP" ] ; then
      echo "$REALTIME - $APPNAME - INFO - gzip decompress $INFILE" >> $LOGFILE
#     gzip does not allow to keep the compressed file. Force this.
      cp -a "$INFILE" $TMPDIR/ 
      gunzip -qfd "$INFILE" >> $ERRLOG 2>&1
      if [ $EXS != 0 ] ; then 
         echo "$REALTIME - $APPNAME - ERROR - gunzip failed with error $EXS"  >> $LOGFILE
         continue # to next file 
      fi  
      mv "$TMPDIR/$(basename $INFILE)" "$(dirname $INFILE)"
   fi
#
#  Inform the calling program of the files created
   echo $MAXIMG $MINIMG $MPAIMG $SNRIMG

#  Plot the images
cat <<EOF > $PLOTFILE
set pm3d map corners2color c1
set title "$STATION $OBSDATE       $DATESTAMP  $TITLE" font "arial,14"
set grid lt 22 lw 1
set xrange [0:24]
set xtics 3
set mxtics 3
set format x "%2.0f"
set xtics font "arial,14"
set xlabel "UT" font "arial,14"
$FSCALE
set yrange $FRQSPAN
set ytics $YTICS font "arial,14"
set mytics $MYTICS
set format y "%2.1f"
#set ytics font "arial,14" font "arial,14"
set ylabel "Frequency [MHz]" font "arial,14"
set zrange [*:*]
set label 1  "VIPIR  $BFN.RIQ" at screen 0.65,0.11 font "arial,10" 
#
# Maximum power plot
set cbrange [$PWRMIN:$PWRMAX]
set cblabel ""
set label 4 "Max [dB]" at screen 0.8,0.85 font "arial,14" 
set term png font arial 12 size $SIZE crop 
set output "$MAXIMG"
splot "$TXTFILE" using (\$1/3600):2:7  title ""
#
# Minimum Power Plot
set cbrange [0:60]
set cblabel ""
set label 4 "Min [dB]" at screen 0.8,0.85 font "arial,14" 
set term png font arial 12 size $SIZE crop 
set output "$MINIMG"
splot "$TXTFILE" using (\$1/3600):2:4  title ""
#
# Noise (MPA) Power Plot
set cbrange [0:60]
set cblabel ""
set label 4 "Noise [dB]" at screen 0.8,0.85 font "arial,14" 
set term png font arial 12 size $SIZE crop 
set output "$MPAIMG"
splot "$TXTFILE" using (\$1/3600):2:6  title ""
#
# SNR Plot
set cbrange [0:$SNRMAX]
set cblabel ""
set label 4 "SNR [dB]" at screen 0.8,0.85 font "arial,14" 
set term png font arial 12 size $SIZE crop 
set output "$SNRIMG"
splot "$TXTFILE" using (\$1/3600):2:(\$7-\$6)  title ""
EOF

   gnuplot $PLOTFILE
done  #   Input file list

exit 0

