#!/bin/bash 
#
# spectrumplot.sh -- Generate Spectrum Analyzer plots
# for Scion HF radar raw data files. 
# Terry Bullett
# University of Colorado
#
# Version 2.05  29Sep10
# useage:
# powerplot-rti2.sh {switches} RIQFILES
#
# Requires:
#    msttcorefonts (included)
#    bc
#    riq-ionogram version 2.05+
#    station_settings.sh
# 
# Version 2.05 -- Moved station-based settings into a separate (sourced) function.
# Version 2.06 -- Remove RIQ file if it was uncompressed 
#              -- Added -x EXTRA output filename feature 
#              -- Added -p OPTIONS for additional pass-through riq_ionogram arguments
#              -- Added -r RNGSPAN to control the range span of the plot
#              -- Added -L for logscale frequency plot
#
export GDFONTPATH=/usr/share/fonts/truetype/msttcorefonts/
#export GDFONTPATH=$HOME/lib/msttcorefonts/
#
APPNAME=spectrumplot
TFMT='%F %T %Z'
REALTIME=$(date -u +"$TFMT" )
YYYY=$(date -u +%Y)
DDD=$(date -u +%j)
LOGFILE="$HOME/log/$APPNAME-$YYYY-$DDD.log"
ERRLOG="$HOME/log/$APPNAME-$YYYY-$DDD.errlog"
TMPDIR="$HOME/tmp/$APPNAME"
mkdir -p $TMPDIR
#
# User-adustable plot parameters
SIZE="1024,768"
FRQSPAN="[0.5:25.0]"
PWRMIN=0
PWRMAX=90
SNRMAX=50
OXMAX=50
#
RIQARGS='-i -sa -r1 30' 
declare -i OBSTIME CLEAN PEAK EIGHT MASK SMASK LINEAR
# Process command line switches
CLEAN=$((0))
NETCDF=$((0))
EIGHT=$((0))
PEAK=$((0))
SMASK=$((2#11111111))
LINEAR=$((1))

while getopts 'a:f:m:chLp:r:t:x:8' OPTION
do
   case $OPTION in
      c) CLEAN=$((1)) 
         ;;
      f) FRQSPAN="$OPTARG"
         ;;
      h)
         echo "spectrumplot Version 2.05"
	 echo "Plot VIPIR data in spectrum analyzer mode"
         echo "Optional switches"
	 echo "-a n  Plot antenna n  (UNIMPLEMENTED, use rxmask)"
         echo "-f FRQSPAN Change the frequency span"
	 echo "-c Clean up by deleting the temp data file"
         echo "-h Print this help message"
         echo "-L log scale (default is linear)"
         echo "-m rxmask   Use rxmask for riq-ionogram"
	 echo "-p OPTIONS Optional additional arguments for riq_ionogram."
	 echo "   Defaults are $RIQARGS"
         echo "-t {title} Add to the title of the plot"
	 echo "-x EXTRA  Adds EXTRA string to the output files"
         echo "-8 Plot all 8 antennas in separate plots"
	 echo ""
	 exit 0 
	 ;;
      L) LINEAR=$((0))
         ;; 
      m) MASK=$(($OPTARG))
         ;;
      p) RIQARGS="$RIQARGS $OPTARG"
         ;; 
      t) TITLE="$OPTARG"
         ;;
      x) EXTRA="$OPTARG"
	 ;;
      8) EIGHT=$((1))
         ;;
   esac
done
#
# Frequency axis scaling
   XTICS='(1.5,2,3,4,5,7,10,15,20)'
   MXICS=''
   FSCALE='set logscale x'
   if (( $LINEAR )) ; then
      XTICS='2'
      FSCALE='unset logscale x'
      MXTICS='10'
   fi
#
# Remove the switches from the list.
shift $((OPTIND - 1))  
#
FILELIST="$@"
for INFILE in $FILELIST ; do 
   REALTIME=$(date -u +"$TFMT" )
   if [ -z $INFILE ] ; then 
      echo "$REALTIME - $APPNAME - FATAL - Input file is undefined" >> $LOGFILE 
      exit -1
   fi
   if [ ! -r $INFILE ] ; then 
      echo "$REALTIME - $APPNAME - FATAL - Input file $INFILE does not exist or is non-readable" >> $LOGFILE 
      exit -2
   fi
#
   if [ -z $OUTDIR ] ; then
      DIR=$(dirname $INFILE)
   else
      DIR=$OUTDIR
   fi
#
#  Determine if the data file is compressed.
#
#
#  Determine if the data file is compressed.
#  ZIP compression is not supported, but it could be.
#  need to preserve the ZIP file.
#
   BZIP2=$(file $INFILE | grep bzip2)
   GZIP=$(file $INFILE | grep gzip)
#
#  Determine file names
   if [ -n "$BZIP2" ] ; then 
      RIQFILE="$(dirname $INFILE)/$(basename $INFILE .bz2)" 
   elif [ -n "$GZIP" ] ; then
      RIQFILE="$(dirname $INFILE)/$(basename $INFILE .gz)"
   else
      RIQFILE="$INFILE"
   fi 
   BFN=$(basename $RIQFILE .RIQ)
#  For v0 data files with no station ID in the filename
   if (( $UOR )) ; then
      URSI="$URSI"
   else 
      URSI=${BFN:0:5}
   fi 

#  Determine the measurement time from the standard filename
   YEAR=${BFN:6:4}
   DAY=${BFN:10:3}
   HH=${BFN:13:2}
   MM=${BFN:15:2}
   SS=${BFN:17:2}
#
   DATESTRING="01Jan$YEAR $DAY days yesterday $HH:$MM:$SS"
   let OBSTIME=$(date -u +%s --date="$DATESTRING") >> $ERRLOG  2>&1  
   EXS=$?
   if [ $EXS != 0 ] ; then 
      echo "$REALTIME - $APPNAME - ERROR - Invalid date $DATESTRING"  >> $LOGFILE
      continue   # to next file
   fi  
   DATESTAMP=$(date -u +'%d%b%y'  --date="$DATESTRING" ) >> $ERRLOG  2>&1
   OBSDATE=$(date -u +'%Y %j %T %Z'  --date="$DATESTRING" ) >> $ERRLOG  2>&1 
#
#  Output and temp files

   REALTIME=$(date -u +"$TFMT" )
   echo "$REALTIME - $APPNAME - INFO - INFILE=$INFILE OUTDIR=$DIR TITLE=$TITLE"  >> $LOGFILE 
#
#  Determine site-specific variables 
   source 'station_settings.sh' 
#
#  Over-ride the receiver mask by command line
   if [ $MASK ] ; then
      RXMASK="-m $MASK"
      BMASK=$( echo  "obase=2;$MASK" | bc)
   else
      RXMASK="-m $SMASK"
      BMASK=$( echo  "obase=2;$SMASK" | bc)
   fi
#
#  pad to 8 places
   BMASK=$(printf "%.8d\n" $BMASK)
#
#   
# decompress as needed
   if [ -n "$BZIP2" ] ; then
      echo "$REALTIME - $APPNAME - INFO - bzip2 decompress $INFILE"  >> $LOGFILE
      bunzip2 -qkfd "$INFILE" >> $ERRLOG 2>&1 
      EXS=$? 
      if [ $EXS != 0 ] ; then 
         echo "$REALTIME - $APPNAME - ERROR - bunzip2 failed with error $EXS"  >> $LOGFILE
         continue # to next file
      fi  
   elif [ -n "$GZIP" ] ; then
      echo "$REALTIME - $APPNAME - INFO - gzip decompress $INFILE" >> $LOGFILE
#     gzip does not allow to keep the compressed file. Force this.
      cp -a "$INFILE" $TMPDIR/ 
      gunzip -qfd "$INFILE" >> $ERRLOG 2>&1
      if [ $EXS != 0 ] ; then 
         echo "$REALTIME - $APPNAME - ERROR - gunzip failed with error $EXS"  >> $LOGFILE
         continue # to next file 
      fi  
      mv "$TMPDIR/$(basename $INFILE)" "$(dirname $INFILE)"
   fi
#
   if (( $EIGHT )) ; then 
      RXLIST='1 2 3 4 5 6 7 8'
      TXT8FILE="$DIR/$BFN$EXTRA"'-all.sam'
      PWR8FILE="$DIR/$BFN$EXTRA"'-all.sam'
      PWR8IMG="$DIR/$BFN$EXTRA"'-all_sam.png'
      if (( $CLEAN )) ; then 
         rm -f $TXT8FILE 
      fi
      for RX in $RXLIST ; do
         TXTFILE="$DIR/$BFN$EXTRA-$RX"'.sam'
         PWRIMG="$DIR/$BFN$EXTRA-$RX"'_sam.png'
         PLOTFILE=$TMPDIR/plot.gnu
         rm -f $PLOTFILE
         if (( $CLEAN )) ; then 
            rm -f $TXTFILE 
         fi
#        Inform the calling program of the files created.
         echo $TXTFILE $PWRIMG 
#        Use the readriq program to extract the plot file from the
#        raw data file.  Many user-maintained options here!
#        This is a long process; do not repeat if already done
         if [ ! -s $TXTFILE  ] ; then 
            if (( $UOR )) ; then 
#              riq_ionogram  -c -U $URSI $RANGE0 $RXNOISE $NCDFLAG $RIQFILE > $TXTFILE
#              Old Wallops  data prior to 2009
               riq_ionogram-0 $RIQARGS  -U $URSI $RXMASK $RIQFILE > $TXTFILE
            else
#              echo riq_ionogram  -c  -a $RX $RANGE0 $RXMASK $RXNOISE $NCDFLAG $RIQFILE
               riq_ionogram  $RIQARGS -a $RX $RANGE0 $RXNOISE $NCDFLAG $RIQFILE > $TXTFILE
            fi 
         fi
         cat < $TXTFILE >> $TXT8FILE
         echo "" >> $TXT8FILE  
         REALTIME=$(date -u +"$TFMT" )
         echo "$REALTIME - $APPNAME - INFO - plotting data from $TXTFILE"  >> $LOGFILE
         cat <<EOF > $PLOTFILE
set grid
set xrange $FRQSPAN
$FSCALE
set xtics $XTICS
set mxtics $MXTICS
set yrange [$PWRMIN:$PWRMAX] 
set ytics 10
set mytics 10   
set title "$BFN $TITLE Receiver $RX"
set xlabel "Frequency [MHz]"
set ylabel "Power [dB]"
set term png font arial 12 size 1024,768 crop 
set output "$PWRIMG"
set label 1  "VIPIR  $BFN.RIQ" at screen 0.70,0.02 font "arial,10" 
# set label 3  "RxMask $BMASK" at screen 0.80,0.95 font "arial,10"
plot "$TXTFILE" using 2:6 title "Noise" with fsteps lt 1 , "$TXTFILE" using 2:7 title "Peak" with fsteps lt 2
set output
exit
EOF
         gnuplot $PLOTFILE
#
         if (( $CLEAN )) ; then 
            rm -f $TXTFILE 
         fi

#         rm -f $PLOTFILE
      done  # each receiver
#
#     Inform the calling program of the files created.
      echo $TXT8FILE $PWR8IMG 
      REALTIME=$(date -u +"$TFMT" )
      echo "$REALTIME - $APPNAME - INFO - plotting data from $TXT8FILE"  >> $LOGFILE

#     Plot all 8 overlayed.
      cat <<EOF > $PLOTFILE
set grid
set xrange $FRQSPAN
$FSCALE
set xtics $XTICS
set mxtics $MXTICS
set yrange [$PWRMIN:$PWRMAX] 
set ytics 10
set mytics 10   
set title "$BFN $TITLE All Receivers"
set xlabel "Frequency [MHz]"
set ylabel "Power [dB]"
set term png font arial 12 size 1024,768 crop 
set output "$PWR8IMG"
set label 1  "VIPIR  $BFN.RIQ" at screen 0.70,0.02 font "arial,10" 
# set label 3  "RxMask $BMASK" at screen 0.80,0.95 font "arial,10"
plot "$TXT8FILE" using 2:6 title "Noise" with fsteps lt 1 , "$TXT8FILE" using 2:7 title "Peak" with fsteps lt 2
set output
exit
EOF
      gnuplot $PLOTFILE
#
#
   else
#     Standard power mode for all 8 receivers summed incoherently
      TXTFILE="$DIR/$BFN$EXTRA"'.sam'
      PWRIMG="$DIR/$BFN$EXTRA"'_sam.png'
      PLOTFILE=$TMPDIR/plot.gnu
      rm -f $PLOTFILE 
      if (( $CLEAN )) ; then 
         rm -f $TXTFILE 
      fi
#     Inform the calling program of the files created.
      echo $TXTFILE $PWRIMG 
#     Use the readriq program to extract the plot file from the
#     raw data file.  Many user-maintained options here!
#     This is a long process; do not repeat if already done
      if [ ! -s $TXTFILE  ] ; then 
         if (( $UOR )) ; then 
#           riq_ionogram  $RIQARGS -U $URSI $RANGE0 $RXMASK $RXNOISE $NCDFLAG $RIQFILE > $TXTFILE
#           Old Wallops  data prior to 2009
            riq_ionogram-0 $RIQARGS -U $URSI $RXMASK $RIQFILE > $TXTFILE
         else
#           echo riq_ionogram  $RIQARGS  $RANGE0 $RXMASK $RXNOISE $NCDFLAG $RIQFILE
            riq_ionogram  $RIQARGS $RANGE0 $RXMASK $RXNOISE $NCDFLAG $RIQFILE > $TXTFILE
         fi 
      fi  
      REALTIME=$(date -u +"$TFMT" )
      echo "$REALTIME - $APPNAME - INFO - plotting data from $TXTFILE"  >> $LOGFILE
      cat <<EOF > $PLOTFILE
set grid 
set xrange $FRQSPAN
$FSCALE
set xtics $XTICS
set mxtics $MXTICS
set yrange [$PWRMIN:$PWRMAX]
set ytics 10
set mytics 10   
set title "$BFN $TITLE"
set xlabel "Frequency [MHz]"
set ylabel 'Power [dB]'
set term png font arial 12 size 1024,768 crop 
set output "$PWRIMG"
set label 1  "VIPIR  $BFN.RIQ" at screen 0.70,0.02 font "arial,10"   
set label 3  "RxMask $BMASK" at screen 0.11,0.02 font "arial,10"
plot "$TXTFILE" using 2:6 title "Noise" with fsteps lt 1 , "$TXTFILE" using 2:7 title "Peak" with fsteps lt 2
set output
exit
EOF
      gnuplot $PLOTFILE
#
      if (( $CLEAN )) ; then 
         rm -f $TXTFILE $TXT8FILE 
      fi
   fi         #  8 or 1 plot 

#
#  Remove original files if decompression was desired
   if [ -n "$BZIP2" ] || [ -n "$GZIP" ] ; then   
      echo "$REALTIME - $APPNAME - INFO - removing decompressed file $RIQFILE"  >> $LOGFILE
      rm  $RIQFILE 2>>$ERRLOG
   fi
#     
   REALTIME=$(date -u +"$TFMT" )
   echo "$REALTIME - $APPNAME - INFO - Finished processing $INFILE"  >> $LOGFILE

   rm -f $PLOTFILE
done   #  Each input file 

exit 0
